
;**************************************************************************
;*                        LCD Display Driver                              *
;*                     for 3x 2x24 LCD Display                            *
;**************************************************************************
;**************************************************************************
;*                      v 1.10 - 06.11.2016                               *
;**************************************************************************
;*          Hardw.: PIC16F1828 used                                       *
;*          OSC.......: Int. 16MHz             POWER.....:   5V extern.   *
;**************************************************************************


 INCLUDE "P16F1828.INC"


_U001			     EQU     B'00100110001010'		;see Page 48 of Datasheet

	;HS oscillator (bit2-0 = 010)
	;MCLR disabled
	;WDT enabled and controlled by SWDTEN bit of register WDTCON (bit4-3 = 01)
	;no protection
	;BOR disabled


_U002			     EQU     B'01100011111111'		;see Page 49 of Datasheet

	;write protection off
	;PLL enabled

_U003			     EQU     B'00100110000100'		;see Page 48 of Datasheet
_U004			     EQU     B'01100011111111'		;see Page 49 of Datasheet



	__CONFIG	_CONFIG1, _U003
	__CONFIG	_CONFIG2, _U004



;---------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------XX
#define		I2C_SLAVE_ADDR	B'10010000'	 ;XX -- 90h
;-------------------------------------------------XX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------



;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"

v_ttt		EQU	H'22'
data_tmp	EQU	H'23'
c_tmp		EQU	H'24'
kbd_buffer	EQU	H'25'
v_tst1		EQU	H'26'
v_tst2		EQU	H'27'
v_tst3		EQU	H'28'


disp_pos	EQU	H'29'

v_act_disp	EQU	H'2A'
		;active display flag:	bit 0 = 1 --> LCD0 is active
		;			bit 1 = 1 --> LCD1 is active
		;			bit 2 = 1 --> LCD2 is active

v_cancel_flag	EQU	H'2B'

v_i2c_byte	EQU	H'2C'

v_row		EQU	H'2D'
v_col		EQU	H'2E'

v_disp_comm	EQU	H'2F'
v_row_wrk	EQU	H'30'
;--------------------------------------------------------------------------
;==========================================================================

		ORG	0		;Reset vector address
		GOTO	init_main	;goto RESET routine when boot.

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************
;
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; initialization
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

init_main	nop

		banksel	OSCCON
		movf	OSCCON,0
		iorlw	B'01111000'
		movwf	OSCCON			;set 16 MHz internal clock (bit 6:3 = 1111) - Datasheet Page 68

		clrf	BSR

		clrf	INTCON			;disable interrupts

		movlw	H'FF'
		movwf	PORTB

		banksel	OPTION_REG		;BANK 1
		movlw	B'01111111'
		movwf	OPTION_REG		;see datasheet Page 176

		movlw	H'03'
		movwf	BSR			;select BANK 3
		clrf	ANSELA
		clrf	ANSELB
		clrf	ANSELC

		movlw	H'01'
		movwf	BSR			;select BANK 1

		movlw	B'11111111'
		movwf	TRISA

		movlw	B'11111111'
		movwf	TRISB

		movlw	B'00000000'
		movwf	TRISC

;no changes required to OSCCON as after RESET the value from Config 1 word will be used (OSCCON - bit 1:bit 0 = 00)

		movlw	H'04'
		movwf	BSR			;select BANK 4
		clrf	WPUA			;disable weak pull-ups for PORTA
		clrf	WPUB			;disable weak pull-ups for PORTB
		clrf	WPUC			;disable weak pull-ups for PORTC
		bsf	WPUA,D'000'		;enable week pull-up for RA0
		bsf	WPUA,D'001'		;enable week pull-up for RA1
		bsf	WPUA,D'002'		;enable week pull-up for RA2

		movlw	H'07'
		movwf	BSR			;select BANK 7
		movlw	H'FF'
		movwf	INLVLA			;ST (1) level or TTL (0) level (?)
		movwf	INLVLB			;ST (1) level or TTL (0) level (?)
		movwf	INLVLC			;ST (1) level or TTL (0) level (?)

		banksel	LATC
		clrf	LATA
		clrf	LATC

		clrf	BSR			;select BANK 0

;comment: no need to change the WDTCON register
;- as reset-value set to WDT to 2 seconds - what is ok for the application
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		movlw	H'FF'
		movwf	PORTB

		movlw	H'FF'
		movwf	v_act_disp

		call	disp_init

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;		banksel	WDTCON
;		bsf	WDTCON,D'000'		;enable Watch-dog
		clrf	BSR			;select BANK 0

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		goto	main_prg

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Main Program
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

main_prg	call	i2c_slave_init

		btfsc	PORTA,D'000'
		call	w_hallo_world

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                             Main Loop
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
main_loop3	call	i2c_slave_init

		banksel	SSP1STAT

start_wait	btfss	SSP1STAT,S
		goto	start_wait

idle_main	nop
		banksel	PIR1
		btfsc	PIR1,SSP1IF
		goto	i2c_add_match
		banksel	SSP1STAT
		btfsc	SSP1STAT,P
		goto	main_loop3		;Stop condition detected
		goto	idle_main


i2c_add_match	nop
		banksel	PIR1
		bcf	PIR1,SSP1IF

		banksel	SSP1STAT

		btfsc	SSP1STAT,D'005'		;Data = 1 / Address = 0
		goto	main_loop3		;we expect address on this stage - but now we got data - something is wrong...

		btfsc	SSP1STAT,D'002'		;Read = 1 / Write = 0
		goto	main_loop3		;we do not support read operation

;we received a matching I2C address with write indication
		banksel	SSP1BUF
		movf	SSP1BUF,0		;read the received address to clear the buffer

;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;now we are in the game... expecting data for 1st position
;in this stage always 4 bytes are expected
;after 4th byte all are written to the video-memory
;reception can be interrupted all the time by generation of stop condition by master
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
receive_loop	call	get_i2c_byte
		btfsc	v_cancel_flag,D'000'
		goto	main_loop3
		movwf	v_row				;Row (0 - 5)

		call	get_i2c_byte
		btfsc	v_cancel_flag,D'000'
		goto	main_loop3
		movwf	v_col				;Column (0 - 23)

receive_loop2	call	set_cursor
		movf	v_disp_comm,0
		call	send_comm

		call	get_i2c_byte
		btfsc	v_cancel_flag,D'000'
		goto	main_loop3
		movwf	v_i2c_byte
		call	send_data

		call	inc_position

		goto	receive_loop2
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
inc_position	incf	v_col,1
		movf	v_col,0
		xorlw	D'024'
		btfss	STATUS,Z
		return
		clrf	v_col
		incf	v_row,1
		return
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
set_cursor	movlw	B'10000000'
		movwf	v_disp_comm
		btfsc	v_row,D'000'
		bsf	v_disp_comm,D'006'		;second row of a LCD
		movf	v_row,0
		andlw	B'11111110'
		movwf	v_row_wrk

		clrf	v_act_disp
		movf	v_row_wrk,0
		xorlw	D'000'
		btfsc	STATUS,Z
		bsf	v_act_disp,D'000'

		movf	v_row_wrk,0
		xorlw	D'002'
		btfsc	STATUS,Z
		bsf	v_act_disp,D'001'

		movf	v_row_wrk,0
		xorlw	D'004'
		btfsc	STATUS,Z
		bsf	v_act_disp,D'002'

		movf	v_act_disp,0
		btfsc	STATUS,Z
		call	clear_row			;row number received was wrong (greater than 5)
;--------------------------------------------------
		movf	v_col,0
		addwf	v_disp_comm,1
		return
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
clear_row	clrf	v_row
		movlw	B'10000000'
		movwf	v_disp_comm
		movlw	B'00000001'
		movwf	v_act_disp
		return
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------
get_i2c_byte	clrf	BSR
		clrf	v_cancel_flag

iddle_data	nop
		banksel	PIR1
		btfsc	PIR1,SSP1IF
		goto	got_it
		banksel	SSP1STAT
		btfsc	SSP1STAT,P
		goto	cancel_comm		;Stop condition detected
		goto	iddle_data

got_it		nop
		banksel	PIR1
		bcf	PIR1,SSP1IF

		banksel	SSP1BUF
		movf	SSP1BUF,0		;read the received data and clear the buffer

		banksel	SSP1STAT

		btfss	SSP1STAT,D'005'		;Data = 1 / Address = 0
		goto	cancel_comm		;we expect data on this stage - just ignore address

		btfsc	SSP1STAT,P
		goto	cancel_comm		;Stop condition has been detected

		clrf	BSR
		return
;-------------------------------------------------
cancel_comm	clrf	BSR
		bsf	v_cancel_flag,D'000'
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_hallo_world	clrf	v_act_disp
		bsf	v_act_disp,D'000'
		call	w_release

		clrf	v_act_disp
		bsf	v_act_disp,D'001'
		call	w_test_d1

		clrf	v_act_disp
		bsf	v_act_disp,D'002'
		call	w_test_d2
		call	dr4
		call	dr4


		movlw	H'FF'
		movwf	v_act_disp

		call	disp_clr1
		call	disp_clr2
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; delay routines
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
d2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return
;------------------------------------------------------------------------------
d2c		MOVLW   D'090'
		MOVWF   TIMER2
cDEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

cDEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    cDEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    cDEL_LOOP1
		return
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------
dr2		call	d2
		call	d2
		call	d2
		return
;------------------------------------
dr4		call	dr2
		call	dr2
		call	dr2
		return


;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x           LCD Display Communication Routines                               x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;RS  = RE2 -- RA5 -- RC4
;R/W = RE1 -- N/C -- N/C
;E   = RE0 -- RA4 -- RC5

;D0 - D3 -- not connected

;D4  = RD0 -- RA0 -- RC0
;D5  = RD1 -- RA1 -- RC1
;D6  = RD2 -- RA2 -- RC2
;D7  = RD3 -- RA3 -- RC3


disp_init	nop

;1st step of init:
;		wait for LCD hardware reset (after power on) - no software reset performed...
		clrf	PORTC
		call	d2_p2

;-- SW Reset to be able to enter 4-bit mode
		bcf	PORTC,D'004'
		bcf	PORTC,D'005'		;EN0 = 0 => disable
		bcf	PORTC,D'006'		;EN1 = 0 => disable
		bcf	PORTC,D'007'		;EN2 = 0 => disable

		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTC,D'005'		;EN0 = 1 => enable
		bsf	PORTC,D'006'		;EN1 = 1 => enable
		bsf	PORTC,D'007'		;EN2 = 1 => enable
		call	dy_comm
		bcf	PORTC,D'005'		;EN0 = 0 => disable
		bcf	PORTC,D'006'		;EN1 = 0 => disable
		bcf	PORTC,D'007'		;EN2 = 0 => disable

		call	d2_p2

		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTC,D'005'		;EN0 = 1 => enable
		bsf	PORTC,D'006'		;EN1 = 1 => enable
		bsf	PORTC,D'007'		;EN2 = 1 => enable
		call	dy_comm
		bcf	PORTC,D'005'		;EN0 = 0 => disable
		bcf	PORTC,D'006'		;EN1 = 0 => disable
		bcf	PORTC,D'007'		;EN2 = 0 => disable

		call	d2_p2

		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTC,D'005'		;EN0 = 1 => enable
		bsf	PORTC,D'006'		;EN1 = 1 => enable
		bsf	PORTC,D'007'		;EN2 = 1 => enable
		call	dy_comm
		bcf	PORTC,D'005'		;EN0 = 0 => disable
		bcf	PORTC,D'006'		;EN1 = 0 => disable
		bcf	PORTC,D'007'		;EN2 = 0 => disable

		call	d2_p2

		movlw	B'00000010'
		call	set_LCD_D
		bsf	PORTC,D'005'		;EN0 = 1 => enable
		bsf	PORTC,D'006'		;EN1 = 1 => enable
		bsf	PORTC,D'007'		;EN2 = 1 => enable
		call	dy_comm
		bcf	PORTC,D'005'		;EN0 = 0 => disable
		bcf	PORTC,D'006'		;EN1 = 0 => disable
		bcf	PORTC,D'007'		;EN2 = 0 => disable

		call	d2_p2

;-----------------------------------------------------------------
;2nd step of init:
;     command:           '001DNFxx' (SET FUNCTION)
;                            D = 1	--> 8 bit interface
;                            D = 0	--> 4 bit interface
;			 '001DNFxx'
;                             N = 1	--> 2 line display
;                             N = 0	--> 1 line display
;			 '001DNFxx'
;                              F = 1	--> 5x10
;			       F = 0	--> 5x7
;			 '001DNFxx'
		movlw	B'00101000'		;2 lines - (for disp 1x16: 1-8  = 00h - 0Fh
;                                                                          9-16 = 40h - 4Fh )
		call	send_comm
;-----------------------------------------------------------------
;3rd step of init:
;     command:           '00001DCB' (DISPLAY ON/OFF)
;                              D = 1	--> Display on
;			       D = 0	--> Display off
;           		 '00001DCB'
;                               C = 1	--> Cursor on
;				C = 0	--> Cursor off
;           		 '00001DCB'
;				 B = 1	--> Blinking on
;				 B = 0	--> Blinking off
		movlw	B'00001000'
		call	send_comm

		movlw	B'00001111'
		movlw	B'00001100'
		call	send_comm
;-----------------------------------------------------------------
;4th step of init:
;     command:           '000001DS' (ENTRY MODE SET)
;				D = 1	--> Increase
;				D = 0	--> Decrease
;		         '000001DS'
;				 S = 1	--> display is shifted
;				 S = 0	--> display is not shifted
;		         '000001DS'
		movlw	B'00000110'
		call	send_comm
;-----------------------------------------------------------------
;clear the display
		movlw	B'00000001'
		call	send_comm
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
set_LCD_D	nop
		movwf	v_ttt

		btfss	v_ttt,D'000'
		bcf	PORTC,D'000'
		call	set_0

		btfss	v_ttt,D'001'
		bcf	PORTC,D'001'
		call	set_1

		btfss	v_ttt,D'002'
		bcf	PORTC,D'002'
		call	set_2

		btfss	v_ttt,D'003'
		bcf	PORTC,D'003'
		call	set_3
		return
;-------------------------------------------
set_0		btfsc	v_ttt,D'000'
		bsf	PORTC,D'000'
		return

set_1		btfsc	v_ttt,D'001'
		bsf	PORTC,D'001'
		return

set_2		btfsc	v_ttt,D'002'
		bsf	PORTC,D'002'
		return

set_3		btfsc	v_ttt,D'003'
		bsf	PORTC,D'003'
		return
;-------------------------------------------
;------------------------------------------------------------------------------------------------
;8-bit data transfer routines
;------------------------------------------------------------------------------------------------
send_data	bcf	PORTC,D'005'		;EN0 -> 0
		bcf	PORTC,D'006'		;EN1 -> 0
		bcf	PORTC,D'007'		;EN2 -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;higher 4 bits are put to RA0-RA3
		call	scbd			;send

		movf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;lower 4 bits are put to RA0-RA3
		call	scbd			;send

		return
;--------------------------------------------------------------------------
;---------------------------------------------------------------------------
;set control bits
scbd		bsf	PORTC,D'004'		;RS -> 1 (data)
;----------------------------------------------------------------
		btfsc	v_act_disp,D'000'
		bsf	PORTC,D'005'
		btfsc	v_act_disp,D'001'
		bsf	PORTC,D'006'
		btfsc	v_act_disp,D'002'
		bsf	PORTC,D'007'
;----------------------------------------------------------------
		call	dy_comm			;wait a while...
		bcf	PORTC,D'005'		;EN0 -> 0
		bcf	PORTC,D'006'		;EN1 -> 0
		bcf	PORTC,D'007'		;EN2 -> 0
		call	d1			;wait a little bit...

		return
;------------------------------------------------------------------------------------------------
send_comm	bcf	PORTC,D'005'		;EN0 -> 0
		bcf	PORTC,D'006'		;EN1 -> 0
		bcf	PORTC,D'007'		;EN2 -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;higher 4 bits are put to RA0-RA3
		call	scbc			;send

		movf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;lower 4 bits are put to RA0-RA3
		call	scbc			;send

		return
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;set control bits
scbc		bcf	PORTC,D'004'		;RS -> 0 (command)
;----------------------------------------------------------------
		btfsc	v_act_disp,D'000'
		bsf	PORTC,D'005'
		btfsc	v_act_disp,D'001'
		bsf	PORTC,D'006'
		btfsc	v_act_disp,D'002'
		bsf	PORTC,D'007'
;----------------------------------------------------------------
		call	dy_comm			;wait a while...
		bcf	PORTC,D'005'		;EN0 -> 0
		bcf	PORTC,D'006'		;EN1 -> 0
		bcf	PORTC,D'007'		;EN2 -> 0
		call	d1			;wait a little bit...
		return
;-----------------------------------------------------------------------------------------------
;----------------------------------------------
dy_comm		call	dy_comm2	;01
		call	dy_comm2	;02
		call	dy_comm2	;03
		call	dy_comm2	;04
		return

dy_comm2	MOVLW   D'255'			;delay value changed: original value was: 010
		MOVWF   TIMER2
DEL_LOOP22c	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP22c
		return
;---------------------------------------------------------------------------
;
d1		call	d1b		;01
		call	d1b		;02
		call	d1b		;02
		call	d1b		;02
		return

d1b		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP11	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP11
		return
;------------------------------------------
d2_p2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOPx1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOPx2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOPx2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOPx1
		return

;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
clear_disp	call	disp_clr1
		call	disp_clr2
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
n9_char		movlw	B'11000000'		;od znaku c.9 treba zmenit adresu na 40h
		call	send_comm

		movlw	B'10000000'
		movwf	disp_pos
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
init2		movlw	B'00000001'
;		call	send_comm
		movlw	B'10000000'
		call	send_comm
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
;character set
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

wm_space	movlw	' '
		call	send_data
		return

wm_dvojbodka	movlw	':'
		call	send_data
		return

wm_point	nop
wm_bodka	movlw	'.'
		call	send_data
		return

wm_hviezdicka	movlw	'*'
		call	send_data
		return

wm_otaznik	movlw	'?'
		call	send_data
		return

wm_sls		movlw	'/'
		call	send_data
		return

wm_kk		movlw	'#'
		call	send_data
		return

wm_minus	movlw	'-'
		call	send_data
		return

wm_plus		movlw	'+'
		call	send_data
		return

wm_hvizdicka	movlw	'*'
		call	send_data
		return

wm_zat1		movlw	'('
		call	send_data
		return

wm_zat2		movlw	')'
		call	send_data
		return

wm_sip1		movlw	'>'
		call	send_data
		return

wm_sip2		movlw	'<'
		call	send_data
		return

wm_stup		movlw	H'DF'
		call	send_data
		return

wm_rs		movlw	'='
		call	send_data
		return

wm_and		movlw	'&'
		call	send_data
		return
;---------- cislice ----------------------------------------
wm_0		movlw	'0'
		call	send_data
		return

wm_1		movlw	'1'
		call	send_data
		return

wm_2		movlw	'2'
		call	send_data
		return

wm_3		movlw	'3'
		call	send_data
		return

wm_4		movlw	'4'
		call	send_data
		return

wm_5		movlw	'5'
		call	send_data
		return

wm_6		movlw	'6'
		call	send_data
		return

wm_7		movlw	'7'
		call	send_data
		return

wm_8		movlw	'8'
		call	send_data
		return

wm_9		movlw	'9'
		call	send_data
		return

;--------- velke pismena -----------------------------------
wm_A		movlw	'A'
		call	send_data
		return

wm_B		movlw	'B'
		call	send_data
		return

wm_C		movlw	'C'
		call	send_data
		return

wm_D		movlw	'D'
		call	send_data
		return

wm_E		movlw	'E'
		call	send_data
		return

wm_F		movlw	'F'
		call	send_data
		return

wm_G		movlw	'G'
		call	send_data
		return

wm_H		movlw	'H'
		call	send_data
		return

wm_I		movlw	'I'
		call	send_data
		return

wm_J		movlw	'J'
		call	send_data
		return

wm_K		movlw	'K'
		call	send_data
		return

wm_L		movlw	'L'
		call	send_data
		return

wm_M		movlw	'M'
		call	send_data
		return

wm_N		movlw	'N'
		call	send_data
		return

wm_O		movlw	'O'
		call	send_data
		return

wm_P		movlw	'P'
		call	send_data
		return

wm_Q		movlw	'Q'
		call	send_data
		return

wm_R		movlw	'R'
		call	send_data
		return

wm_S		movlw	'S'
		call	send_data
		return

wm_T		movlw	'T'
		call	send_data
		return

wm_U		movlw	'U'
		call	send_data
		return

wm_V		movlw	'V'
		call	send_data
		return

wm_W		movlw	'W'
		call	send_data
		return

wm_X		movlw	'X'
		call	send_data
		return

wm_Y		movlw	'Y'
		call	send_data
		return

wm_Z		movlw	'Z'
		call	send_data
		return

;--------- male pismena -----------------------------------

wm_a		movlw	'a'
		call	send_data
		return

wm_b		movlw	'b'
		call	send_data
		return

wm_c		movlw	'c'
		call	send_data
		return

wm_d		movlw	'd'
		call	send_data
		return

wm_e		movlw	'e'
		call	send_data
		return

wm_f		movlw	'f'
		call	send_data
		return

wm_g		movlw	'g'
		call	send_data
		return

wm_h		movlw	'h'
		call	send_data
		return

wm_i		movlw	'i'
		call	send_data
		return

wm_k		movlw	'k'
		call	send_data
		return

wm_l		movlw	'l'
		call	send_data
		return

wm_m		movlw	'm'
		call	send_data
		return

wm_n		movlw	'n'
		call	send_data
		return

wm_o		movlw	'o'
		call	send_data
		return

wm_p		movlw	'p'
		call	send_data
		return

wm_r		movlw	'r'
		call	send_data
		return

wm_s		movlw	's'
		call	send_data
		return

wm_t		movlw	't'
		call	send_data
		return

wm_u		movlw	'u'
		call	send_data
		return

wm_v		movlw	'v'
		call	send_data
		return

wm_w		movlw	'w'
		call	send_data
		return

wm_x		movlw	'x'
		call	send_data
		return

wm_y		movlw	'y'
		call	send_data
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


;---------------------------------------------------------------------------
disp_clr1	movlw	B'10000000'		;line 1 / char0
		call	send_comm

disp_clrx	call	wm_space	;01
		call	wm_space	;02
		call	wm_space	;03
		call	wm_space	;04
		call	wm_space	;05
		call	wm_space	;06
		call	wm_space	;07
		call	wm_space	;08
		call	wm_space	;09
		call	wm_space	;10
		call	wm_space	;11
		call	wm_space	;12
		call	wm_space	;13
		call	wm_space	;14
		call	wm_space	;15
		call	wm_space	;16
		call	wm_space	;17
		call	wm_space	;18
		call	wm_space	;19
		call	wm_space	;20
		call	wm_space	;21
		call	wm_space	;22
		call	wm_space	;23
		call	wm_space	;24

		return
;---------------------------------------------------------------------------
disp_clr2	movlw	B'11000000'		;line 2 / char0
		call	send_comm

		goto	disp_clrx

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
convert2ascii	movwf	c_tmp
;
		xorlw	H'15'
		btfsc	STATUS,Z
		goto	wm_Q
;
		movf	c_tmp,0
		xorlw	H'1D'
		btfsc	STATUS,Z
		goto	wm_W
;
		movf	c_tmp,0
		xorlw	H'24'
		btfsc	STATUS,Z
		goto	wm_E
;
		movf	c_tmp,0
		xorlw	H'2D'
		btfsc	STATUS,Z
		goto	wm_R
;
		movf	c_tmp,0
		xorlw	H'2C'
		btfsc	STATUS,Z
		goto	wm_T
;
		movf	c_tmp,0
		xorlw	H'35'
		btfsc	STATUS,Z
		goto	wm_Y
;
		movf	c_tmp,0
		xorlw	H'3C'
		btfsc	STATUS,Z
		goto	wm_U
;
		movf	c_tmp,0
		xorlw	H'43'
		btfsc	STATUS,Z
		goto	wm_I
;
		movf	c_tmp,0
		xorlw	H'44'
		btfsc	STATUS,Z
		goto	wm_O
;
		movf	c_tmp,0
		xorlw	H'4D'
		btfsc	STATUS,Z
		goto	wm_P
;
		movf	c_tmp,0
		xorlw	H'16'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'1E'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'26'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'25'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'2E'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'36'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'3D'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'3E'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'46'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'45'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'1C'
		btfsc	STATUS,Z
		goto	wm_A
;
		movf	c_tmp,0
		xorlw	H'1B'
		btfsc	STATUS,Z
		goto	wm_S
;
		movf	c_tmp,0
		xorlw	H'23'
		btfsc	STATUS,Z
		goto	wm_D
;
		movf	c_tmp,0
		xorlw	H'2B'
		btfsc	STATUS,Z
		goto	wm_F
;
		movf	c_tmp,0
		xorlw	H'34'
		btfsc	STATUS,Z
		goto	wm_G
;
		movf	c_tmp,0
		xorlw	H'33'
		btfsc	STATUS,Z
		goto	wm_H
;
		movf	c_tmp,0
		xorlw	H'3B'
		btfsc	STATUS,Z
		goto	wm_J
;
		movf	c_tmp,0
		xorlw	H'42'
		btfsc	STATUS,Z
		goto	wm_K
;
		movf	c_tmp,0
		xorlw	H'4B'
		btfsc	STATUS,Z
		goto	wm_L
;
		movf	c_tmp,0
		xorlw	H'1A'
		btfsc	STATUS,Z
		goto	wm_Z
;
		movf	c_tmp,0
		xorlw	H'22'
		btfsc	STATUS,Z
		goto	wm_X
;
		movf	c_tmp,0
		xorlw	H'21'
		btfsc	STATUS,Z
		goto	wm_C
;
		movf	c_tmp,0
		xorlw	H'2A'
		btfsc	STATUS,Z
		goto	wm_V
;
		movf	c_tmp,0
		xorlw	H'32'
		btfsc	STATUS,Z
		goto	wm_B
;
		movf	c_tmp,0
		xorlw	H'31'
		btfsc	STATUS,Z
		goto	wm_N
;
		movf	c_tmp,0
		xorlw	H'3A'
		btfsc	STATUS,Z
		goto	wm_M
;
		movf	c_tmp,0
		xorlw	H'29'
		btfsc	STATUS,Z
		goto	wm_space
;
		movf	c_tmp,0
		xorlw	H'6C'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'75'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'7D'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'6B'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'73'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'74'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'69'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'72'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'7A'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'70'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'71'
		btfsc	STATUS,Z
		goto	wm_bodka
;
		movf	c_tmp,0
		xorlw	H'79'
		btfsc	STATUS,Z
		goto	wm_plus
;
		movf	c_tmp,0
		xorlw	H'7B'
		btfsc	STATUS,Z
		goto	wm_minus
;
		movf	c_tmp,0
		xorlw	H'7C'
		btfsc	STATUS,Z
		goto	wm_hvizdicka

		movf	c_tmp,0
		xorlw	H'76'			;ESC
		btfsc	STATUS,Z
;!!!!!!		goto	main_s


;not known character - either transmission error or not decoded... - do nothing...
		decf	disp_pos,1
		
		return


		return


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
; Firmware Release
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;-------------------------------------------------------------------------------
w_release	nop

		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_B
		call	wm_i
		call	wm_g
		call	wm_space
		call	wm_minus
		call	wm_space
		call	wm_L
		call	wm_C
		call	wm_D

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_F
		call	wm_i
		call	wm_r
		call	wm_m
		call	wm_w
		call	wm_a
		call	wm_r
		call	wm_e
		call	wm_space
		call	wm_1
		call	wm_bodka
		call	wm_0
		call	wm_9
		call	wm_minus
		call	wm_1
		call	wm_8
		call	wm_bodka
		call	wm_0
		call	wm_5
		call	wm_bodka
		call	wm_2
		call	wm_0
		call	wm_1
		call	wm_6

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_test_d1	nop

		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_I
		call	wm_n
		call	wm_t
		call	wm_e
		call	wm_r
		call	wm_f
		call	wm_a
		call	wm_c
		call	wm_e
		call	wm_space
		call	wm_1
		call	wm_dvojbodka
		call	wm_space
		call	wm_I
		call	wm_2
		call	wm_C

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_I
		call	wm_n
		call	wm_t
		call	wm_e
		call	wm_r
		call	wm_f
		call	wm_a
		call	wm_c
		call	wm_e
		call	wm_space
		call	wm_2
		call	wm_dvojbodka
		call	wm_space
		call	wm_S
		call	wm_P
		call	wm_I

		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_test_d2	nop

		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_A
		call	wm_c
		call	wm_t
		call	wm_i
		call	wm_v
		call	wm_e
		call	wm_dvojbodka
		call	wm_space
		call	wm_I
		call	wm_2
		call	wm_C

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_minus
		call	wm_minus
		call	wm_minus
		call	wm_minus
		call	wm_minus
		call	wm_minus
		call	wm_minus
		call	wm_minus
		call	wm_minus
		call	wm_minus
		call	wm_minus
		call	wm_minus
		call	wm_minus
		call	wm_minus
		call	wm_minus
		call	wm_minus

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_bin7	movlw	'0'
		btfsc	kbd_buffer,D'007'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'006'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'005'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'004'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'003'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'002'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'001'
		movlw	'1'
		call	send_data

		call	wm_x

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
write_bin	movlw	'0'
		btfsc	kbd_buffer,D'007'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'006'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'005'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'004'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'003'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'002'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'001'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'000'
		movlw	'1'
		call	send_data

		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_dec	movlw	'0'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

conv_next	movf	kbd_buffer,0
		btfsc	STATUS,Z
		goto	tst_conv_f

		incf	v_tst3,1
		movf	v_tst3,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wdcn

		movlw	'0'
		movwf	v_tst3
		incf	v_tst2,1
		movf	v_tst2,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wdcn

		movlw	'0'
		movwf	v_tst2
		incf	v_tst1,1

wdcn		decf	kbd_buffer,1
		goto	conv_next
;-------------------------------------------------
tst_conv_f	movf	v_tst1,0
;;;		call	send_data		;do not write first digit
		movf	v_tst2,0
		call	send_data
		movf	v_tst3,0
		call	send_data
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_hex	nop

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'00'
		btfsc	STATUS,Z
		call	wm_0

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'10'
		btfsc	STATUS,Z
		call	wm_1

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'20'
		btfsc	STATUS,Z
		call	wm_2

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'30'
		btfsc	STATUS,Z
		call	wm_3

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'40'
		btfsc	STATUS,Z
		call	wm_4

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'50'
		btfsc	STATUS,Z
		call	wm_5

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'60'
		btfsc	STATUS,Z
		call	wm_6

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'70'
		btfsc	STATUS,Z
		call	wm_7

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'80'
		btfsc	STATUS,Z
		call	wm_8

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'90'
		btfsc	STATUS,Z
		call	wm_9

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'A0'
		btfsc	STATUS,Z
		call	wm_A

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'B0'
		btfsc	STATUS,Z
		call	wm_B

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'C0'
		btfsc	STATUS,Z
		call	wm_C

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'D0'
		btfsc	STATUS,Z
		call	wm_D

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'E0'
		btfsc	STATUS,Z
		call	wm_E

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'F0'
		btfsc	STATUS,Z
		call	wm_F
;
		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'00'
		btfsc	STATUS,Z
		goto	wm_0

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	wm_1

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	wm_2

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	wm_3

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'04'
		btfsc	STATUS,Z
		goto	wm_4

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'05'
		btfsc	STATUS,Z
		goto	wm_5

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'06'
		btfsc	STATUS,Z
		goto	wm_6

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'07'
		btfsc	STATUS,Z
		goto	wm_7

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	wm_8

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'09'
		btfsc	STATUS,Z
		goto	wm_9

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0A'
		btfsc	STATUS,Z
		goto	wm_A

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0B'
		btfsc	STATUS,Z
		goto	wm_B

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0C'
		btfsc	STATUS,Z
		goto	wm_C

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0D'
		btfsc	STATUS,Z
		goto	wm_D

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0E'
		btfsc	STATUS,Z
		goto	wm_E

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0F'
		btfsc	STATUS,Z
		goto	wm_F


		return

;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------


;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;I2C
;set up I2C BUS for communication as a slave
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
i2c_slave_init	movlw	H'04'
		movwf	BSR			;select BANK 4

		movlw	B'00110110'
		movwf	SSP1CON1		;See Page 280 of Datasheet

;set-up the slave address
		movlw	I2C_SLAVE_ADDR		;define the I2C slave address
		movwf	SSP1ADD
		clrf	SSP1STAT

		clrf	BSR			;select BANK 0
		return
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------


;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


		END